#include "NativeSignAPI.h"
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <dnne.h>
#include <chrono>


using namespace std;
//using namespace std::chrono_literals;

#ifdef _WIN32
const char* pathToLib = "SigSignAPI.WrapperNE.dll";
#elif __APPLE__
const char* pathToLib = "SigSignAPI.WrapperNE.dylib";
#elif __linux__
const char* pathToLib = "SigSignAPI.WrapperNE.so";
#endif

typedef intptr_t** (DNNE_CALLTYPE* PtrPtr_ft)();
typedef int(DNNE_CALLTYPE* Free_t)(intptr_t*);
typedef int(DNNE_CALLTYPE* SetDevice_t)(intptr_t*);
typedef intptr_t**(DNNE_CALLTYPE* Int_PtrPtr_t)(int32_t);
typedef int(DNNE_CALLTYPE* charPtr_Int_t)(char*);
typedef int(DNNE_CALLTYPE* LoadPdf_t)(intptr_t*);
typedef int(DNNE_CALLTYPE* Int_ft)();
typedef intptr_t(DNNE_CALLTYPE* int_int_intPtr_t)(int32_t, int32_t);
typedef int32_t(DNNE_CALLTYPE* intPtr_int_int_int_t)(intptr_t, int32_t, int32_t);
typedef int32_t(DNNE_CALLTYPE* int_int_int_int_t)(int32_t, int32_t, int32_t);
typedef int32_t(DNNE_CALLTYPE* AddSignatureField_t)(intptr_t, double, double, double, double, int32_t);
typedef intptr_t(DNNE_CALLTYPE* intptr_intptr_ft)(intptr_t);
typedef int32_t(DNNE_CALLTYPE* intptr_int_ft)(intptr_t);
typedef int32_t(DNNE_CALLTYPE* SignSignatureField_t)(intptr_t, intptr_t, intptr_t);
typedef intptr_t(DNNE_CALLTYPE* Intptr_t)();
typedef int32_t(DNNE_CALLTYPE* SetBehavioursignInDocument_t)(intptr_t, intptr_t, int32_t);
typedef intptr_t(DNNE_CALLTYPE* GetDocSignRectangleFromTopLeftAndZoomLevel_t)(int32_t, int32_t, int32_t);
typedef int(DNNE_CALLTYPE* IntptrInt_t)(intptr_t);
typedef int(DNNE_CALLTYPE* FreeAll_t)();
typedef int32_t(DNNE_CALLTYPE* int_int_t)(int32_t);
typedef int32_t(DNNE_CALLTYPE* DialogStartSignature_t)(intptr_t, int32_t);



#ifdef _WIN32
 void* NativeSignAPI::load_library(const char* path)
{
	return LoadLibraryA(path);
}
intptr_t NativeSignAPI::get_export(void* h, const char* name)
{
	FARPROC f = GetProcAddress((HMODULE)h, name);
	return (intptr_t)f;
}
#else 
 #include <dlfcn.h>
#include <limits.h>
#include <unistd.h>
 void* NativeSignAPI::load_library(const char* path)
{
	void* h = dlopen(path, RTLD_LAZY | RTLD_LOCAL);
	return h;
}
intptr_t NativeSignAPI::get_export(void* h, const char* name)
{
	void* f = dlsym(h, name);
	return (intptr_t)f;
}
#endif

typedef intptr_t (DNNE_CALLTYPE* Int32_intptr_t)(int32_t);

void *NativeSignAPI::NativeSignAPIAPIobj = NULL;
bool NativeSignAPI::IsDeviceSet = false;

int NativeSignAPI::Free(intptr_t* ptr)
{
	Free_t free2 = (Free_t)get_export(NativeSignAPIAPIobj, "Free");
	//RETURN_FAIL_IF_FALSE(free, "Failed to get GetHello export\n");
	return free2(ptr);
}
int NativeSignAPI::FreeAll()
{

	FreeAll_t freeAll = (FreeAll_t)get_export(NativeSignAPIAPIobj, "FreeAll");
	if (!freeAll) 
		return NULL;
	return freeAll();
}


 bool NativeSignAPI::load_library() 
{
	NativeSignAPI::IsDeviceSet = false;
	NativeSignAPIAPIobj = load_library(pathToLib);
	if(NativeSignAPIAPIobj == NULL)
		return false;
	return true;
}


std::vector<std::string> NativeSignAPI::EnumerateDevicesFilter(unsigned int filter) 
 {
	 Int_PtrPtr_t EnumerateDevicesfilter = (Int_PtrPtr_t)get_export(NativeSignAPIAPIobj, "EnumerateDevicesFilter");
	 vector<string> devices = vector<string>();
	 if (!EnumerateDevicesfilter)
	 {
		 // Add Log Here
		 return devices;
	 } 

	 intptr_t** deviceNames = EnumerateDevicesfilter((int32_t )filter);
	 
	 if(deviceNames == NULL)
	 {
		 // Add Log Here
		 return devices;
	 } 
	
	long number = (size_t)deviceNames[0];
	intptr_t** entries = &deviceNames[1];
	for (int k = 0; k < number; k++)
		devices.push_back(std::string((char*)entries[k], std::allocator<char>()));

	return devices;
 }



 int NativeSignAPI::SetDevice(std::string device) 
 {
	SetDevice_t setDevice = (SetDevice_t)get_export(NativeSignAPIAPIobj, "SetDevice");
	if (!setDevice) 
		return NULL;

	int error = setDevice((intptr_t*)device.c_str());

	if(error == 0)
		IsDeviceSet = true;

	return error;
}

 int NativeSignAPI::LoadPdf(char* path) 
 {
	LoadPdf_t loadpdf = (LoadPdf_t)get_export(NativeSignAPIAPIobj, "LoadPdf");
	if (!loadpdf) 
		return NULL;

	char* lengthPrefixedDocumentOctets = loadLengthPrefixedOctetsFromPath(path);

	return loadpdf((intptr_t*)(lengthPrefixedDocumentOctets));
}


 char*  NativeSignAPI::loadLengthPrefixedOctetsFromPath(const char* path) 
 {
	FILE* fp = fopen(path, "rb");

	fpos_t start;
	fgetpos(fp, &start);

	fseek(fp, 0, SEEK_END);
	int32_t size = ftell(fp);
	fsetpos(fp, &start);

	char* buffer = new char[4 + size];
	((int32_t*)buffer)[0] = size;
	fread(&buffer[4], 1, size, fp);

	fclose(fp);
	return buffer;
}

int NativeSignAPI::GetNumberOfPages()
{
	Int_ft getNumberOfPages = (Int_ft)get_export(NativeSignAPIAPIobj, "GetNumberOfPages");
	if (!getNumberOfPages) 
		return NULL;
	return getNumberOfPages();
}

intptr_t NativeSignAPI::RenderPageWithDpiAsPng(int32_t page, int32_t dpi)
{
	int_int_intPtr_t RenderPagewithDpiAsPng = (int_int_intPtr_t)get_export(NativeSignAPIAPIobj, "RenderPageWithDpiAsPng");
	if (!RenderPagewithDpiAsPng) 
		return NULL;
	return RenderPagewithDpiAsPng(page, dpi);
}

int32_t NativeSignAPI::SetDocumentViewing(intptr_t image, int32_t pageNumber, int32_t numberOfPages)
{
	intPtr_int_int_int_t SetDocumentviewing = (intPtr_int_int_int_t)get_export(NativeSignAPIAPIobj, "SetDocumentViewing");
	if (!SetDocumentviewing) 
		return NULL;
	return SetDocumentviewing(image, pageNumber, numberOfPages);
}

int32_t NativeSignAPI::ViewDocument(int32_t pageNumber, int32_t zoomLevel, int32_t startSignatureButtonEnabled)
{
	int_int_int_int_t Viewdocument = (int_int_int_int_t)get_export(NativeSignAPIAPIobj, "ViewDocument");
	if (!Viewdocument) 
		return NULL;
	return Viewdocument(pageNumber, zoomLevel, startSignatureButtonEnabled);
}

int32_t NativeSignAPI::AddSignatureField(intptr_t fieldName, double row0, double column0, double row1, double column1, int32_t pageNumber)
{
	AddSignatureField_t AddSignaturefield = (AddSignatureField_t)get_export(NativeSignAPIAPIobj, "AddSignatureField");
	if (!AddSignaturefield) 
		return NULL;
	return AddSignaturefield(fieldName, row0, column0, row1, column1, pageNumber);
}

intptr_t** NativeSignAPI::GetAcroFormFields()
{
	PtrPtr_ft GetAcroformFields = (PtrPtr_ft)get_export(NativeSignAPIAPIobj, "GetAcroFormFields");
	if (!GetAcroformFields) 
		return NULL;
	return GetAcroformFields();
}

intptr_t NativeSignAPI::GetFieldName(intptr_t fieldHandle)
{
	intptr_intptr_ft GetfieldName = (intptr_intptr_ft)get_export(NativeSignAPIAPIobj, "GetFieldName");
	if (!GetfieldName) 
		return NULL;
	return GetfieldName(fieldHandle);
}

int32_t NativeSignAPI::GetFieldIsSignatureField(intptr_t fieldHandle)
{
	intptr_int_ft GetFieldIsSignaturefield = (intptr_int_ft)get_export(NativeSignAPIAPIobj, "GetFieldIsSignatureField");
	if (!GetFieldIsSignaturefield) 
		return NULL;
	return GetFieldIsSignaturefield(fieldHandle);
}

int32_t NativeSignAPI::SignSignatureField(intptr_t signatureField, intptr_t signatureInfo, intptr_t behaviour)
{
	SignSignatureField_t SignSignaturefield = (SignSignatureField_t)get_export(NativeSignAPIAPIobj, "SignSignatureField");
	if (!SignSignaturefield) 
		return NULL;
	return SignSignaturefield(signatureField, signatureInfo, behaviour);
}

intptr_t NativeSignAPI::GetBehaviourDefault(void)
{
	Intptr_t GetBehaviourdefault = (Intptr_t)get_export(NativeSignAPIAPIobj, "GetBehaviourDefault");
	if (!GetBehaviourdefault) 
		return NULL;
	return GetBehaviourdefault();
}

int32_t NativeSignAPI::SetBehaviourSignInDocument(intptr_t behaviour, intptr_t docSignRectangle, int32_t crop)
{
	SetBehavioursignInDocument_t SetBehavioursignInDocument = (SetBehavioursignInDocument_t)get_export(NativeSignAPIAPIobj, "SetBehaviourSignInDocument");
	if (!SetBehavioursignInDocument) 
		return NULL;
	return SetBehavioursignInDocument(behaviour, docSignRectangle, crop);
}

intptr_t NativeSignAPI::GetDocSignRectangleFromTopLeftAndZoomLevel(int32_t top, int32_t left, int32_t zoomLevel)
{
	GetDocSignRectangleFromTopLeftAndZoomLevel_t GetDocSignRectangleFromTopLeftAndzoomLevel = (GetDocSignRectangleFromTopLeftAndZoomLevel_t)get_export(NativeSignAPIAPIobj, "GetDocSignRectangleFromTopLeftAndZoomLevel");
	if (!GetDocSignRectangleFromTopLeftAndzoomLevel) 
		return NULL;
	return GetDocSignRectangleFromTopLeftAndzoomLevel(top, left, zoomLevel);
}

int32_t NativeSignAPI::SetButtonCallback( intptr_t fptr)
 {
	 IntptrInt_t SetButtoncallback = (IntptrInt_t)get_export(NativeSignAPIAPIobj, "SetButtonCallback");
	if (!SetButtoncallback) 
		return NULL;
	return SetButtoncallback(fptr);
 }

intptr_t NativeSignAPI::DownloadPdf(void)
{
	Intptr_t Downloadpdf = (Intptr_t)get_export(NativeSignAPIAPIobj, "DownloadPdf");
	if (!Downloadpdf) 
		return NULL;
	return Downloadpdf();
}

int32_t NativeSignAPI::DialogCreate(int32_t pageNumber)
{
	int_int_t Dialogcreate = (int_int_t)get_export(NativeSignAPIAPIobj, "DialogCreate");
	if (!Dialogcreate) 
		return NULL;
	return Dialogcreate(pageNumber);
}

int32_t NativeSignAPI::DialogStartSignature(intptr_t path, int32_t scrollToSignature)
{
	DialogStartSignature_t DialogStartsignature = (DialogStartSignature_t)get_export(NativeSignAPIAPIobj, "DialogStartSignature");
	if (!DialogStartsignature) 
		return NULL;
	return DialogStartsignature(path, scrollToSignature);
}

intptr_t NativeSignAPI::DialogSave(intptr_t path)
{
	intptr_int_ft Dialogsave = (intptr_int_ft)get_export(NativeSignAPIAPIobj, "DialogSave");
	if (!Dialogsave) 
		return NULL;
	return Dialogsave(path);
}




 
 

